@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012 Red Hat, Inc.
+ * Copyright © 2012, 2013 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,7 +25,7 @@
 #ifndef PHOTOS_BASE_ITEM_H
 #define PHOTOS_BASE_ITEM_H
 
-#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gegl.h>
 #include <gio/gio.h>
 #include <glib-object.h>
 #include <gtk/gtk.h>
@@ -68,7 +68,7 @@ struct _PhotosBaseItemClass
 {
   GObjectClass parent_class;
 
-  GdkPixbuf *(*load) (PhotosBaseItem *self, GCancellable *cancellable, GError **error);
+  GeglNode *(*load) (PhotosBaseItem *self, GCancellable *cancellable, GError **error);
   void (*set_favorite) (PhotosBaseItem *self, gboolean favorite);
   void (*update_type_description) (PhotosBaseItem *self);
 
@@ -84,6 +84,8 @@ void                photos_base_item_destroy            (PhotosBaseItem *self);
 
 const gchar        *photos_base_item_get_author         (PhotosBaseItem *self);
 
+GeglRectangle       photos_base_item_get_bbox           (PhotosBaseItem *self);
+
 gint64              photos_base_item_get_date_created   (PhotosBaseItem *self);
 
 const gchar        *photos_base_item_get_default_app_name (PhotosBaseItem *self);
@@ -92,12 +94,18 @@ GdkPixbuf          *photos_base_item_get_icon           (PhotosBaseItem *self);
 
 const gchar        *photos_base_item_get_id             (PhotosBaseItem *self);
 
+const gchar        *photos_base_item_get_identifier     (PhotosBaseItem *self);
+
 const gchar        *photos_base_item_get_mime_type      (PhotosBaseItem *self);
 
 gint64              photos_base_item_get_mtime          (PhotosBaseItem *self);
 
 const gchar        *photos_base_item_get_name           (PhotosBaseItem *self);
 
+GdkPixbuf          *photos_base_item_get_pristine_icon  (PhotosBaseItem *self);
+
+const gchar        *photos_base_item_get_resource_urn   (PhotosBaseItem *self);
+
 const gchar        *photos_base_item_get_type_description (PhotosBaseItem *self);
 
 const gchar        *photos_base_item_get_uri            (PhotosBaseItem *self);
@@ -113,7 +121,7 @@ void                photos_base_item_load_async         (PhotosBaseItem *self,
                                                          GAsyncReadyCallback callback,
                                                          gpointer user_data);
 
-GdkPixbuf          *photos_base_item_load_finish        (PhotosBaseItem *self, GAsyncResult *res, GError **error);
+GeglNode           *photos_base_item_load_finish        (PhotosBaseItem *self, GAsyncResult *res, GError **error);
 
 void                photos_base_item_open               (PhotosBaseItem *self,
                                                          GdkScreen *screen,
@@ -127,6 +135,8 @@ void                photos_base_item_set_default_app_name (PhotosBaseItem *self,
 
 void                photos_base_item_set_favorite       (PhotosBaseItem *self, gboolean favorite);
 
+void                photos_base_item_trash              (PhotosBaseItem *self);
+
 G_END_DECLS
 
 #endif /* PHOTOS_BASE_ITEM_H */
