@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012 Red Hat, Inc.
+ * Copyright © 2012, 2013 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,13 +25,14 @@
 
 #include "config.h"
 
-#include <clutter-gtk/clutter-gtk.h>
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <libgd/gd.h>
 
+#include "photos-application.h"
 #include "photos-collection-manager.h"
+#include "photos-header-bar.h"
 #include "photos-item-manager.h"
 #include "photos-main-toolbar.h"
 #include "photos-mode-controller.h"
@@ -41,10 +42,10 @@
 
 struct _PhotosMainToolbarPrivate
 {
-  ClutterActor *actor;
-  ClutterLayoutManager *layout;
+  GSimpleAction *gear_menu;
   GtkWidget *coll_back_button;
-  GtkWidget *widget;
+  GtkWidget *selection_menu;
+  GtkWidget *toolbar;
   PhotosBaseManager *col_mngr;
   PhotosBaseManager *item_mngr;
   PhotosBaseManager *src_mngr;
@@ -59,45 +60,7 @@ struct _PhotosMainToolbarPrivate
 };
 
 
-G_DEFINE_TYPE (PhotosMainToolbar, photos_main_toolbar, G_TYPE_OBJECT);
-
-
-static void
-photos_main_toolbar_favorites_toggled (GtkToggleButton *toggle_button, gpointer user_data)
-{
-  PhotosMainToolbar *self = PHOTOS_MAIN_TOOLBAR (user_data);
-
-  if (gtk_toggle_button_get_active (toggle_button))
-    photos_mode_controller_set_window_mode (self->priv->mode_cntrlr, PHOTOS_WINDOW_MODE_FAVORITES);
-}
-
-
-static void
-photos_main_toolbar_overview_toggled (GtkToggleButton *toggle_button, gpointer user_data)
-{
-  PhotosMainToolbar *self = PHOTOS_MAIN_TOOLBAR (user_data);
-
-  if (gtk_toggle_button_get_active (toggle_button))
-    photos_mode_controller_set_window_mode (self->priv->mode_cntrlr, PHOTOS_WINDOW_MODE_OVERVIEW);
-}
-
-
-static void
-photos_main_toolbar_add_modes (PhotosMainToolbar *self, PhotosWindowMode window_mode)
-{
-  PhotosMainToolbarPrivate *priv = self->priv;
-  GtkWidget *button;
-
-  button = gd_main_toolbar_add_mode (GD_MAIN_TOOLBAR (priv->widget), _("Photos"));
-  if (window_mode == PHOTOS_WINDOW_MODE_OVERVIEW)
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-  g_signal_connect (button, "toggled", G_CALLBACK (photos_main_toolbar_overview_toggled), self);
-
-  button = gd_main_toolbar_add_mode (GD_MAIN_TOOLBAR (priv->widget), _("Favorites"));
-  if (window_mode == PHOTOS_WINDOW_MODE_FAVORITES)
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-  g_signal_connect (button, "toggled", G_CALLBACK (photos_main_toolbar_favorites_toggled), self);
-}
+G_DEFINE_TYPE (PhotosMainToolbar, photos_main_toolbar, GTK_TYPE_BOX);
 
 
 static void
@@ -107,14 +70,15 @@ photos_main_toolbar_set_toolbar_title (PhotosMainToolbar *self)
   GObject *active_collection;
   PhotosWindowMode window_mode;
   gboolean selection_mode;
-  gchar *detail = NULL;
   gchar *primary = NULL;
 
   active_collection = photos_base_manager_get_active_object (priv->col_mngr);
   selection_mode = photos_selection_controller_get_selection_mode (priv->sel_cntrlr);
   window_mode = photos_mode_controller_get_window_mode (priv->mode_cntrlr);
 
-  if (window_mode == PHOTOS_WINDOW_MODE_OVERVIEW)
+  if (window_mode == PHOTOS_WINDOW_MODE_OVERVIEW
+      || window_mode == PHOTOS_WINDOW_MODE_COLLECTIONS
+      || window_mode == PHOTOS_WINDOW_MODE_FAVORITES)
     {
       if (!selection_mode)
         {
@@ -127,24 +91,29 @@ photos_main_toolbar_set_toolbar_title (PhotosMainToolbar *self)
       else
         {
           GList *selection;
+          gchar *label;
           guint length;
 
           selection = photos_selection_controller_get_selection (priv->sel_cntrlr);
           length = g_list_length (selection);
           if (length == 0)
-            detail = g_strdup(_("Click on items to select them"));
+            label = g_strdup(_("Click on items to select them"));
           else
-            detail = g_strdup_printf (_("%d selected"), length);
+            label = g_strdup_printf (_("%d selected"), length);
 
           if (active_collection != NULL)
             {
-              primary = g_strdup (photos_base_item_get_name (PHOTOS_BASE_ITEM (active_collection)));
+              primary = g_strdup_printf ("<b>%s</b> (%s)",
+                                         photos_base_item_get_name (PHOTOS_BASE_ITEM (active_collection)),
+                                         label);
             }
           else
             {
-              primary = detail;
-              detail = NULL;
+              primary = label;
+              label = NULL;
             }
+
+          g_free (label);
         }
     }
   else if (window_mode == PHOTOS_WINDOW_MODE_PREVIEW)
@@ -155,18 +124,32 @@ photos_main_toolbar_set_toolbar_title (PhotosMainToolbar *self)
       primary = g_strdup (photos_base_item_get_name (PHOTOS_BASE_ITEM (item)));
     }
 
-  if (detail != NULL)
-    {
-      gchar *tmp;
+  if (selection_mode)
+    gd_header_button_set_label (GD_HEADER_BUTTON (priv->selection_menu), primary);
+  else
+    gd_header_bar_set_title (GD_HEADER_BAR (priv->toolbar), primary);
 
-      tmp = detail;
-      detail = g_strconcat ("(", detail, ")", NULL);
-      g_free (tmp);
-    }
-
-  gd_main_toolbar_set_labels (GD_MAIN_TOOLBAR (priv->widget), primary, detail);
   g_free (primary);
-  g_free (detail);
+}
+
+
+static GtkWidget *
+photos_main_toolbar_add_back_button (PhotosMainToolbar *self)
+{
+  PhotosMainToolbarPrivate *priv = self->priv;
+  GtkTextDirection direction;
+  GtkWidget *back_button;
+  const gchar *icon_name;
+
+  direction = gtk_widget_get_direction (GTK_WIDGET (priv->toolbar));
+  icon_name = (direction == GTK_TEXT_DIR_RTL) ? "go-next-symbolic" : "go-previous-symbolic";
+
+  back_button = gd_header_simple_button_new ();
+  gd_header_button_set_label (GD_HEADER_BUTTON (back_button), _("Back"));
+  gd_header_button_set_symbolic_icon_name (GD_HEADER_BUTTON (back_button), icon_name);
+  gd_header_bar_pack_start (GD_HEADER_BAR (priv->toolbar), back_button);
+
+  return back_button;
 }
 
 
@@ -174,7 +157,7 @@ static void
 photos_main_toolbar_coll_back_button_clicked (GtkButton *button, gpointer user_data)
 {
   PhotosMainToolbar *self = PHOTOS_MAIN_TOOLBAR (user_data);
-  photos_base_manager_set_active_object (self->priv->col_mngr, NULL);
+  photos_item_manager_activate_previous_collection (PHOTOS_ITEM_MANAGER (self->priv->item_mngr));
 }
 
 
@@ -184,29 +167,65 @@ photos_main_toolbar_active_changed (PhotosBaseManager *manager, GObject *object,
   PhotosMainToolbar *self = PHOTOS_MAIN_TOOLBAR (user_data);
   PhotosMainToolbarPrivate *priv = self->priv;
   GObject *item;
+  PhotosHeaderBarMode mode;
 
   item = photos_base_manager_get_active_object (priv->col_mngr);
-  if (item != NULL && priv->coll_back_button == NULL)
+  if (item != NULL)
     {
-      priv->coll_back_button = gd_main_toolbar_add_button (GD_MAIN_TOOLBAR (priv->widget),
-                                                           "go-previous-symbolic",
-                                                           _("Back"),
-                                                           TRUE);
-      g_signal_connect (priv->coll_back_button,
-                        "clicked",
-                        G_CALLBACK (photos_main_toolbar_coll_back_button_clicked),
-                        self);
+      mode = PHOTOS_HEADER_BAR_MODE_STANDALONE;
+      if (priv->coll_back_button == NULL)
+        {
+          priv->coll_back_button = photos_main_toolbar_add_back_button (self);
+          gtk_widget_show (priv->coll_back_button);
+
+          g_signal_connect (priv->coll_back_button,
+                            "clicked",
+                            G_CALLBACK (photos_main_toolbar_coll_back_button_clicked),
+                            self);
+        }
     }
-  else if (item == NULL && priv->coll_back_button != NULL)
+  else
     {
-      gtk_widget_destroy (priv->coll_back_button);
-      priv->coll_back_button = NULL;
+      mode = PHOTOS_HEADER_BAR_MODE_NORMAL;
+      if (priv->coll_back_button != NULL)
+        {
+          gtk_widget_destroy (priv->coll_back_button);
+          priv->coll_back_button = NULL;
+        }
     }
 
+  photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), mode);
   photos_main_toolbar_set_toolbar_title (self);
 }
 
 
+static void
+photos_main_toolbar_select_button_clicked (GtkButton *button, gpointer user_data)
+{
+  PhotosMainToolbar *self = PHOTOS_MAIN_TOOLBAR (user_data);
+  photos_selection_controller_set_selection_mode (self->priv->sel_cntrlr, TRUE);
+}
+
+
+static void
+photos_main_toolbar_add_selection_button (PhotosMainToolbar *self)
+{
+  PhotosMainToolbarPrivate *priv = self->priv;
+  GtkWidget *selection_button;
+
+  selection_button = gd_header_simple_button_new ();
+  gd_header_button_set_label (GD_HEADER_BUTTON (selection_button), _("Select Items"));
+  gd_header_button_set_symbolic_icon_name (GD_HEADER_BUTTON (selection_button), "object-select-symbolic");
+  gd_header_bar_pack_end (GD_HEADER_BAR (priv->toolbar), selection_button);
+  g_signal_connect (selection_button, "clicked", G_CALLBACK (photos_main_toolbar_select_button_clicked), self);
+
+  priv->collection_id = g_signal_connect (priv->col_mngr,
+                                          "active-changed",
+                                          G_CALLBACK (photos_main_toolbar_active_changed),
+                                          self);
+}
+
+
 static void
 photos_main_toolbar_back_button_clicked (GtkButton *button, gpointer user_data)
 {
@@ -223,6 +242,12 @@ photos_main_toolbar_clear_state_data (PhotosMainToolbar *self)
 {
   PhotosMainToolbarPrivate *priv = self->priv;
 
+  if (priv->coll_back_button != NULL)
+    {
+      gtk_widget_destroy (priv->coll_back_button);
+      priv->coll_back_button = NULL;
+    }
+
   if (priv->collection_id != 0)
     {
       g_signal_handler_disconnect (priv->col_mngr, priv->collection_id);
@@ -241,41 +266,46 @@ static void
 photos_main_toolbar_clear_toolbar (PhotosMainToolbar *self)
 {
   PhotosMainToolbarPrivate *priv = self->priv;
-  GtkStyleContext *context;
 
   photos_main_toolbar_clear_state_data (self);
-  context = gtk_widget_get_style_context (priv->widget);
-  gtk_style_context_remove_class (context, "selection-mode");
-  gtk_widget_reset_style (priv->widget);
-  gd_main_toolbar_clear (GD_MAIN_TOOLBAR (priv->widget));
+  photos_header_bar_clear (PHOTOS_HEADER_BAR (priv->toolbar));
+  g_simple_action_set_enabled (priv->gear_menu, FALSE);
 }
 
 
-static void
-photos_main_toolbar_destroy (GtkWidget *widget, gpointer user_data)
+static GMenu *
+photos_main_toolbar_create_preview_menu (PhotosMainToolbar *self)
 {
-  PhotosMainToolbar *self = PHOTOS_MAIN_TOOLBAR (user_data);
-  PhotosMainToolbarPrivate *priv = self->priv;
+  GMenu *menu;
+  GtkBuilder *builder;
+  PhotosBaseItem *item;
 
-  photos_main_toolbar_clear_state_data (self);
+  builder = gtk_builder_new ();
+  gtk_builder_add_from_resource (builder, "/org/gnome/photos/preview-menu.ui", NULL);
 
-  if (priv->window_mode_id != 0)
+  menu = G_MENU (g_object_ref (gtk_builder_get_object (builder, "preview-menu")));
+  item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (self->priv->item_mngr));
+  if (item != NULL)
     {
-      g_signal_handler_disconnect (priv->mode_cntrlr, priv->window_mode_id);
-      priv->window_mode_id = 0;
+      const gchar *default_app_name;
+
+      default_app_name = photos_base_item_get_default_app_name (item);
+      if (default_app_name != NULL)
+        {
+          GMenu *section;
+          gchar *label;
+
+          section = G_MENU (gtk_builder_get_object (builder, "open-section"));
+          g_menu_remove (section, 0);
+
+          label = g_strdup_printf (_("Open with %s"), default_app_name);
+          g_menu_prepend (section, label, "app.open-current");
+          g_free (label);
+        }
     }
 
-  if (priv->selection_mode_id != 0)
-    {
-      g_signal_handler_disconnect (priv->sel_cntrlr, priv->selection_mode_id);
-      priv->selection_mode_id = 0;
-    }
-
-  if (priv->search_source_id != 0)
-    {
-      g_signal_handler_disconnect (priv->src_mngr, priv->search_source_id);
-      priv->search_source_id = 0;
-    }
+  g_object_unref (builder);
+  return menu;
 }
 
 
@@ -288,10 +318,16 @@ photos_main_toolbar_done_button_clicked (GtkButton *button, gpointer user_data)
 
 
 static void
-photos_main_toolbar_select_button_clicked (GtkButton *button, gpointer user_data)
+photos_main_toolbar_populate_for_collections (PhotosMainToolbar *self)
 {
-  PhotosMainToolbar *self = PHOTOS_MAIN_TOOLBAR (user_data);
-  photos_selection_controller_set_selection_mode (self->priv->sel_cntrlr, TRUE);
+  PhotosMainToolbarPrivate *priv = self->priv;
+  GObject *object;
+
+  photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), PHOTOS_HEADER_BAR_MODE_NORMAL);
+  photos_main_toolbar_add_selection_button (self);
+
+  object = photos_base_manager_get_active_object (priv->col_mngr);
+  photos_main_toolbar_active_changed (priv->col_mngr, object, self);
 }
 
 
@@ -300,21 +336,9 @@ photos_main_toolbar_populate_for_favorites (PhotosMainToolbar *self)
 {
   PhotosMainToolbarPrivate *priv = self->priv;
   GObject *object;
-  GtkWidget *selection_button;
 
-  gd_main_toolbar_set_show_modes (GD_MAIN_TOOLBAR (priv->widget), TRUE);
-  photos_main_toolbar_add_modes (self, PHOTOS_WINDOW_MODE_FAVORITES);
-
-  selection_button = gd_main_toolbar_add_button (GD_MAIN_TOOLBAR (priv->widget),
-                                                 "object-select-symbolic",
-                                                 _("Select Items"),
-                                                 FALSE);
-  g_signal_connect (selection_button, "clicked", G_CALLBACK (photos_main_toolbar_select_button_clicked), self);
-
-  priv->collection_id = g_signal_connect (priv->col_mngr,
-                                          "active-changed",
-                                          G_CALLBACK (photos_main_toolbar_active_changed),
-                                          self);
+  photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), PHOTOS_HEADER_BAR_MODE_NORMAL);
+  photos_main_toolbar_add_selection_button (self);
 
   object = photos_base_manager_get_active_object (priv->col_mngr);
   photos_main_toolbar_active_changed (priv->col_mngr, object, self);
@@ -326,21 +350,9 @@ photos_main_toolbar_populate_for_overview (PhotosMainToolbar *self)
 {
   PhotosMainToolbarPrivate *priv = self->priv;
   GObject *object;
-  GtkWidget *selection_button;
 
-  gd_main_toolbar_set_show_modes (GD_MAIN_TOOLBAR (priv->widget), TRUE);
-  photos_main_toolbar_add_modes (self, PHOTOS_WINDOW_MODE_OVERVIEW);
-
-  selection_button = gd_main_toolbar_add_button (GD_MAIN_TOOLBAR (priv->widget),
-                                                 "object-select-symbolic",
-                                                 _("Select Items"),
-                                                 FALSE);
-  g_signal_connect (selection_button, "clicked", G_CALLBACK (photos_main_toolbar_select_button_clicked), self);
-
-  priv->collection_id = g_signal_connect (priv->col_mngr,
-                                          "active-changed",
-                                          G_CALLBACK (photos_main_toolbar_active_changed),
-                                          self);
+  photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), PHOTOS_HEADER_BAR_MODE_NORMAL);
+  photos_main_toolbar_add_selection_button (self);
 
   object = photos_base_manager_get_active_object (priv->col_mngr);
   photos_main_toolbar_active_changed (priv->col_mngr, object, self);
@@ -351,16 +363,23 @@ static void
 photos_main_toolbar_populate_for_preview (PhotosMainToolbar *self)
 {
   PhotosMainToolbarPrivate *priv = self->priv;
-  GtkTextDirection direction;
+  GMenu *preview_menu;
   GtkWidget *back_button;
-  const gchar *icon_name;
+  GtkWidget *menu_button;
 
-  gd_main_toolbar_set_show_modes (GD_MAIN_TOOLBAR (priv->widget), FALSE);
+  photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), PHOTOS_HEADER_BAR_MODE_STANDALONE);
 
-  direction = gtk_widget_get_direction (GTK_WIDGET (priv->widget));
-  icon_name = (direction == GTK_TEXT_DIR_RTL) ? "go-next-symbolic" : "go-previous-symbolic";
-  back_button = gd_main_toolbar_add_button (GD_MAIN_TOOLBAR (priv->widget), icon_name, _("Back"), TRUE);
+  back_button = photos_main_toolbar_add_back_button (self);
   g_signal_connect (back_button, "clicked", G_CALLBACK (photos_main_toolbar_back_button_clicked), self);
+
+  preview_menu = photos_main_toolbar_create_preview_menu (self);
+  menu_button = gd_header_menu_button_new ();
+  gtk_actionable_set_action_name (GTK_ACTIONABLE (menu_button), "app.gear-menu");
+  gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (menu_button), G_MENU_MODEL (preview_menu));
+  gd_header_button_set_symbolic_icon_name (GD_HEADER_BUTTON (menu_button), "emblem-system-symbolic");
+  gd_header_bar_pack_end (GD_HEADER_BAR (priv->toolbar), menu_button);
+
+  g_simple_action_set_enabled (priv->gear_menu, TRUE);
 }
 
 
@@ -371,13 +390,11 @@ photos_main_toolbar_populate_for_selection_mode (PhotosMainToolbar *self)
   GtkStyleContext *context;
   GtkWidget *selection_button;
 
-  gd_main_toolbar_set_show_modes (GD_MAIN_TOOLBAR (priv->widget), FALSE);
+  photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), PHOTOS_HEADER_BAR_MODE_SELECTION);
 
-  context = gtk_widget_get_style_context (priv->widget);
-  gtk_style_context_add_class (context, "selection-mode");
-  gtk_widget_reset_style (priv->widget);
-
-  selection_button = gd_main_toolbar_add_button (GD_MAIN_TOOLBAR (priv->widget), NULL, _("Done"), FALSE);
+  selection_button = gd_header_simple_button_new ();
+  gd_header_button_set_label (GD_HEADER_BUTTON (selection_button), _("Done"));
+  gd_header_bar_pack_end (GD_HEADER_BAR (priv->toolbar), selection_button);
   context = gtk_widget_get_style_context (selection_button);
   gtk_style_context_add_class (context, "suggested-action");
   g_signal_connect (selection_button, "clicked", G_CALLBACK (photos_main_toolbar_done_button_clicked), self);
@@ -402,6 +419,8 @@ photos_main_toolbar_reset_toolbar_mode (PhotosMainToolbar *self)
 
   if (selection_mode)
     photos_main_toolbar_populate_for_selection_mode (self);
+  else if (window_mode == PHOTOS_WINDOW_MODE_COLLECTIONS)
+    photos_main_toolbar_populate_for_collections (self);
   else if (window_mode == PHOTOS_WINDOW_MODE_FAVORITES)
     photos_main_toolbar_populate_for_favorites (self);
   else if (window_mode == PHOTOS_WINDOW_MODE_OVERVIEW)
@@ -410,7 +429,7 @@ photos_main_toolbar_reset_toolbar_mode (PhotosMainToolbar *self)
     photos_main_toolbar_populate_for_preview (self);
 
   photos_main_toolbar_set_toolbar_title (self);
-  gtk_widget_show_all (priv->widget);
+  gtk_widget_show_all (priv->toolbar);
 }
 
 
@@ -428,6 +447,26 @@ photos_main_toolbar_dispose (GObject *object)
   PhotosMainToolbar *self = PHOTOS_MAIN_TOOLBAR (object);
   PhotosMainToolbarPrivate *priv = self->priv;
 
+  photos_main_toolbar_clear_state_data (self);
+
+  if (priv->window_mode_id != 0)
+    {
+      g_signal_handler_disconnect (priv->mode_cntrlr, priv->window_mode_id);
+      priv->window_mode_id = 0;
+    }
+
+  if (priv->selection_mode_id != 0)
+    {
+      g_signal_handler_disconnect (priv->sel_cntrlr, priv->selection_mode_id);
+      priv->selection_mode_id = 0;
+    }
+
+  if (priv->search_source_id != 0)
+    {
+      g_signal_handler_disconnect (priv->src_mngr, priv->search_source_id);
+      priv->search_source_id = 0;
+    }
+
   g_clear_object (&priv->col_mngr);
   g_clear_object (&priv->item_mngr);
   g_clear_object (&priv->src_mngr);
@@ -442,32 +481,35 @@ static void
 photos_main_toolbar_init (PhotosMainToolbar *self)
 {
   PhotosMainToolbarPrivate *priv;
-  ClutterActor *actor;
-  GtkStyleContext *context;
+  GMenu *selection_menu;
+  GtkApplication *app;
+  GtkBuilder *builder;
 
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, PHOTOS_TYPE_MAIN_TOOLBAR, PhotosMainToolbarPrivate);
   priv = self->priv;
 
-  priv->widget = gd_main_toolbar_new ();
-  gtk_toolbar_set_icon_size (GTK_TOOLBAR (priv->widget), GTK_ICON_SIZE_MENU);
-  context = gtk_widget_get_style_context (priv->widget);
-  gtk_style_context_add_class (context, GTK_STYLE_CLASS_MENUBAR);
-  gtk_widget_show (priv->widget);
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (self), GTK_ORIENTATION_VERTICAL);
+  gtk_widget_show (GTK_WIDGET (self));
 
-  priv->layout = clutter_box_layout_new ();
-  clutter_box_layout_set_orientation (CLUTTER_BOX_LAYOUT (priv->layout), CLUTTER_ORIENTATION_VERTICAL);
+  priv->toolbar = photos_header_bar_new ();
+  gtk_container_add (GTK_CONTAINER (self), priv->toolbar);
+  gtk_widget_show (priv->toolbar);
 
-  priv->actor = clutter_actor_new ();
-  clutter_actor_set_layout_manager (priv->actor, priv->layout);
+  app = photos_application_new ();
+  priv->gear_menu = G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (app), "gear-menu"));
+  g_object_unref (app);
 
-  actor = gtk_clutter_actor_new_with_contents (priv->widget);
-  clutter_box_layout_pack (CLUTTER_BOX_LAYOUT (priv->layout),
-                           actor,
-                           FALSE,
-                           TRUE,
-                           FALSE,
-                           CLUTTER_BOX_ALIGNMENT_CENTER,
-                           CLUTTER_BOX_ALIGNMENT_START);
+  builder = gtk_builder_new ();
+  gtk_builder_add_from_resource (builder, "/org/gnome/photos/selection-menu.ui", NULL);
+
+  selection_menu = G_MENU (gtk_builder_get_object (builder, "selection-menu"));
+  priv->selection_menu = gd_header_menu_button_new ();
+  gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (priv->selection_menu), G_MENU_MODEL (selection_menu));
+  gd_header_button_set_use_markup (GD_HEADER_BUTTON (priv->selection_menu), TRUE);
+  g_object_unref (builder);
+
+  photos_header_bar_set_selection_menu (PHOTOS_HEADER_BAR (priv->toolbar),
+                                        GD_HEADER_BUTTON (priv->selection_menu));
 
   priv->col_mngr = photos_collection_manager_new ();
   priv->item_mngr = photos_item_manager_new ();
@@ -492,8 +534,6 @@ photos_main_toolbar_init (PhotosMainToolbar *self)
                                                       self);
 
   photos_main_toolbar_reset_toolbar_mode (self);
-
-  g_signal_connect (priv->widget, "destroy", G_CALLBACK (photos_main_toolbar_destroy), self);
 }
 
 
@@ -508,15 +548,15 @@ photos_main_toolbar_class_init (PhotosMainToolbarClass *class)
 }
 
 
-PhotosMainToolbar *
+GtkWidget *
 photos_main_toolbar_new (void)
 {
   return g_object_new (PHOTOS_TYPE_MAIN_TOOLBAR, NULL);
 }
 
 
-ClutterActor *
-photos_main_toolbar_get_actor (PhotosMainToolbar *self)
+void
+photos_main_toolbar_set_stack (PhotosMainToolbar *self, GdStack *stack)
 {
-  return self->priv->actor;
+  photos_header_bar_set_stack (PHOTOS_HEADER_BAR (self->priv->toolbar), stack);
 }
