@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012 Red Hat, Inc.
+ * Copyright © 2012, 2013 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,12 +25,15 @@
 
 #include "config.h"
 
+#include "photos-base-item.h"
 #include "photos-collection-manager.h"
+#include "photos-fetch-collection-state-job.h"
 #include "photos-organize-collection-model.h"
 
 
 struct _PhotosOrganizeCollectionModelPrivate
 {
+  GtkTreePath *coll_path;
   GtkTreeRowReference *placeholder_ref;
   PhotosBaseManager *manager;
   gulong coll_added_id;
@@ -41,15 +44,126 @@ struct _PhotosOrganizeCollectionModelPrivate
 G_DEFINE_TYPE (PhotosOrganizeCollectionModel, photos_organize_collection_model, GTK_TYPE_LIST_STORE);
 
 
+static gboolean
+photos_organize_collection_model_foreach (GtkTreeModel *model,
+                                          GtkTreePath *path,
+                                          GtkTreeIter *iter,
+                                          gpointer user_data)
+{
+  PhotosOrganizeCollectionModel *self = PHOTOS_ORGANIZE_COLLECTION_MODEL (model);
+  PhotosBaseItem *collection = PHOTOS_BASE_ITEM (user_data);
+  gboolean ret_val = FALSE;
+  gchar *id;
+
+  gtk_tree_model_get (GTK_TREE_MODEL (self), iter, PHOTOS_ORGANIZE_MODEL_ID, &id, -1);
+  if (g_strcmp0 (photos_base_item_get_id (collection), id) == 0)
+    {
+      self->priv->coll_path = gtk_tree_path_copy (path);
+      ret_val = TRUE;
+      goto out;
+    }
+
+ out:
+  g_free (id);
+  return ret_val;
+}
+
+
+static GtkTreeIter *
+photos_organize_collection_model_find_collection_iter (PhotosOrganizeCollectionModel *self,
+                                                       PhotosBaseItem *collection)
+{
+  PhotosOrganizeCollectionModelPrivate *priv = self->priv;
+
+  gtk_tree_model_foreach (GTK_TREE_MODEL (self), photos_organize_collection_model_foreach, collection);
+  if (priv->coll_path != NULL)
+    {
+      GtkTreeIter iter;
+
+      gtk_tree_model_get_iter (GTK_TREE_MODEL (self), &iter, priv->coll_path);
+      g_clear_pointer (&priv->coll_path, (GDestroyNotify) gtk_tree_path_free);
+      return gtk_tree_iter_copy (&iter);
+    }
+
+  return NULL;
+}
+
+
+static void
+photos_organize_collection_model_fetch_collection_state_executed (GHashTable *collection_state, gpointer user_data)
+{
+  PhotosOrganizeCollectionModel *self = PHOTOS_ORGANIZE_COLLECTION_MODEL (user_data);
+  GHashTableIter iter;
+  const gchar *idx;
+  gpointer value;
+
+  photos_organize_collection_model_remove_placeholder (self);
+
+  g_hash_table_iter_init (&iter, collection_state);
+  while (g_hash_table_iter_next (&iter, (gpointer) &idx, (gpointer) &value))
+    {
+      GtkTreeIter *iter;
+      PhotosBaseItem *collection;
+      gint state = GPOINTER_TO_INT (value);
+
+      if (state & PHOTOS_COLLECTION_STATE_HIDDEN)
+        continue;
+
+      collection = PHOTOS_BASE_ITEM (photos_base_manager_get_object_by_id (self->priv->manager, idx));
+      iter = photos_organize_collection_model_find_collection_iter (self, collection);
+      if (iter == NULL)
+        {
+          GtkTreeIter tmp;
+
+          gtk_list_store_append (GTK_LIST_STORE (self), &tmp);
+          iter = gtk_tree_iter_copy (&tmp);
+        }
+
+      gtk_list_store_set (GTK_LIST_STORE (self),
+                          iter,
+                          PHOTOS_ORGANIZE_MODEL_ID, idx,
+                          PHOTOS_ORGANIZE_MODEL_NAME, photos_base_item_get_name (collection),
+                          PHOTOS_ORGANIZE_MODEL_STATE, state,
+                          -1);
+      gtk_tree_iter_free (iter);
+    }
+
+  g_object_unref (self);
+}
+
+
+static void
+photos_organize_collection_model_refresh_state (PhotosOrganizeCollectionModel *self)
+{
+  PhotosFetchCollectionStateJob *job;
+
+  job = photos_fetch_collection_state_job_new ();
+  photos_fetch_collection_state_job_run (job,
+                                         photos_organize_collection_model_fetch_collection_state_executed,
+                                         g_object_ref (self));
+}
+
+
 static void
 photos_organize_collection_model_object_added (PhotosBaseManager *manager, GObject *object, gpointer user_data)
 {
+  PhotosOrganizeCollectionModel *self = PHOTOS_ORGANIZE_COLLECTION_MODEL (user_data);
+  photos_organize_collection_model_refresh_state (self);
 }
 
 
 static void
 photos_organize_collection_model_object_removed (PhotosBaseManager *manager, GObject *object, gpointer user_data)
 {
+  PhotosOrganizeCollectionModel *self = PHOTOS_ORGANIZE_COLLECTION_MODEL (user_data);
+  GtkTreeIter *iter;
+
+  iter = photos_organize_collection_model_find_collection_iter (self, PHOTOS_BASE_ITEM (object));
+  if (iter == NULL)
+    return;
+
+  gtk_list_store_remove (GTK_LIST_STORE (self), iter);
+  gtk_tree_iter_free (iter);
 }
 
 
@@ -59,6 +173,18 @@ photos_organize_collection_model_dispose (GObject *object)
   PhotosOrganizeCollectionModel *self = PHOTOS_ORGANIZE_COLLECTION_MODEL (object);
   PhotosOrganizeCollectionModelPrivate *priv = self->priv;
 
+  if (priv->coll_added_id != 0)
+    {
+      g_signal_handler_disconnect (priv->manager, priv->coll_added_id);
+      priv->coll_added_id = 0;
+    }
+
+  if (priv->coll_removed_id != 0)
+    {
+      g_signal_handler_disconnect (priv->manager, priv->coll_removed_id);
+      priv->coll_removed_id = 0;
+    }
+
   g_clear_object (&priv->manager);
 
   G_OBJECT_CLASS (photos_organize_collection_model_parent_class)->dispose (object);
@@ -69,8 +195,10 @@ static void
 photos_organize_collection_model_finalize (GObject *object)
 {
   PhotosOrganizeCollectionModel *self = PHOTOS_ORGANIZE_COLLECTION_MODEL (object);
+  PhotosOrganizeCollectionModelPrivate *priv = self->priv;
 
-  gtk_tree_row_reference_free (self->priv->placeholder_ref);
+  gtk_tree_path_free (priv->coll_path);
+  gtk_tree_row_reference_free (priv->placeholder_ref);
 
   G_OBJECT_CLASS (photos_organize_collection_model_parent_class)->finalize (object);
 }
@@ -101,7 +229,8 @@ photos_organize_collection_model_init (PhotosOrganizeCollectionModel *self)
                                             G_CALLBACK (photos_organize_collection_model_object_removed),
                                             self);
 
-  /* TODO: populate the model */
+  /* Populate the model. */
+  photos_organize_collection_model_refresh_state (self);
 }
 
 
@@ -138,7 +267,7 @@ photos_organize_collection_model_add_placeholder (PhotosOrganizeCollectionModel
                       &iter,
                       PHOTOS_ORGANIZE_MODEL_ID, PHOTOS_COLLECTION_PLACEHOLDER_ID,
                       PHOTOS_ORGANIZE_MODEL_NAME, "",
-                      PHOTOS_ORGANIZE_MODEL_STATE, PHOTOS_ORGANIZE_COLLECTION_STATE_ACTIVE,
+                      PHOTOS_ORGANIZE_MODEL_STATE, PHOTOS_COLLECTION_STATE_ACTIVE,
                       -1);
 
   placeholder_path = gtk_tree_model_get_path (GTK_TREE_MODEL (self), &iter);
@@ -149,25 +278,6 @@ photos_organize_collection_model_add_placeholder (PhotosOrganizeCollectionModel
 }
 
 
-void
-photos_organize_collection_model_destroy (PhotosOrganizeCollectionModel *self)
-{
-  PhotosOrganizeCollectionModelPrivate *priv = self->priv;
-
-  if (priv->coll_added_id != 0)
-    {
-      g_signal_handler_disconnect (priv->manager, priv->coll_added_id);
-      priv->coll_added_id = 0;
-    }
-
-  if (priv->coll_removed_id != 0)
-    {
-      g_signal_handler_disconnect (priv->manager, priv->coll_removed_id);
-      priv->coll_removed_id = 0;
-    }
-}
-
-
 GtkTreePath *
 photos_organize_collection_model_get_placeholder (PhotosOrganizeCollectionModel *self, gboolean forget)
 {
@@ -187,6 +297,13 @@ photos_organize_collection_model_get_placeholder (PhotosOrganizeCollectionModel
 }
 
 
+void
+photos_organize_collection_model_refresh_collection_state (PhotosOrganizeCollectionModel *self)
+{
+  photos_organize_collection_model_refresh_state (self);
+}
+
+
 void
 photos_organize_collection_model_remove_placeholder (PhotosOrganizeCollectionModel *self)
 {
