@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012 Red Hat, Inc.
+ * Copyright © 2012, 2013 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,10 +26,12 @@
 #include "config.h"
 #include <libgd/gd.h>
 
+#include "photos-application.h"
 #include "photos-enums.h"
 #include "photos-item-manager.h"
 #include "photos-load-more-button.h"
 #include "photos-selection-controller.h"
+#include "photos-tracker-collections-controller.h"
 #include "photos-tracker-favorites-controller.h"
 #include "photos-tracker-overview-controller.h"
 #include "photos-utils.h"
@@ -230,10 +232,12 @@ photos_view_container_window_mode_changed (PhotosModeController *mode_cntrlr,
 {
   PhotosViewContainer *self = PHOTOS_VIEW_CONTAINER (user_data);
 
-  if (mode == PHOTOS_WINDOW_MODE_OVERVIEW)
+  photos_view_container_disconnect_view (self);
+
+  if (mode == PHOTOS_WINDOW_MODE_COLLECTIONS
+      || mode == PHOTOS_WINDOW_MODE_FAVORITES
+      || mode == PHOTOS_WINDOW_MODE_OVERVIEW)
     photos_view_container_connect_view (self);
-  else
-    photos_view_container_disconnect_view (self);
 }
 
 
@@ -242,6 +246,8 @@ photos_view_container_constructed (GObject *object)
 {
   PhotosViewContainer *self = PHOTOS_VIEW_CONTAINER (object);
   PhotosViewContainerPrivate *priv = self->priv;
+  GAction *action;
+  GtkApplication *app;
   gboolean status;
 
   G_OBJECT_CLASS (photos_view_container_parent_class)->constructed (object);
@@ -287,6 +293,10 @@ photos_view_container_constructed (GObject *object)
 
   switch (priv->mode)
     {
+    case PHOTOS_WINDOW_MODE_COLLECTIONS:
+      priv->trk_cntrlr = photos_tracker_collections_controller_new ();
+      break;
+
     case PHOTOS_WINDOW_MODE_FAVORITES:
       priv->trk_cntrlr = photos_tracker_favorites_controller_new ();
       break;
@@ -300,6 +310,16 @@ photos_view_container_constructed (GObject *object)
       break;
     }
 
+  app = photos_application_new ();
+
+  action = g_action_map_lookup_action (G_ACTION_MAP (app), "select-all");
+  g_signal_connect_swapped (action, "activate", G_CALLBACK (gd_main_view_select_all), priv->view);
+
+  action = g_action_map_lookup_action (G_ACTION_MAP (app), "select-none");
+  g_signal_connect_swapped (action, "activate", G_CALLBACK (gd_main_view_unselect_all), priv->view);
+
+  g_object_unref (app);
+
   g_signal_connect (priv->trk_cntrlr,
                     "query-status-changed",
                     G_CALLBACK (photos_view_container_query_status_changed),
